
import os
from pylab import *
from enthought.mayavi import mlab
from pyclaw.plotters.data import ClawPlotData
from setplot import setplot
from html_movie import make_movie

from pyclaw.data import Data
geodata = Data('setgeo.data')
Rearth = geodata.Rearth
from mapper import latlong

def plotframe(frameno,figno=260,level=5):

    plotdata = ClawPlotData()
    plotdata.outdir = "_output_%i" % figno
    print "Plotting solution from ",plotdata.outdir
    plotdata = setplot(plotdata)
    try:
        frame = plotdata.getframe(frameno)
    except:
        print "Unable to get frame"
        return
    mlab.figure(figno,bgcolor=(1,1,1))
    mlab.clf()
    for grid in frame.grids:
        if grid.level == level:

            y = grid.c_center[1]
            # scale x by cos(y) to adjust for sphere:
            x = grid.c_center[0] * cos(pi*y/180.)
            q = grid.q
            eta = q[:,:,3]
            h = q[:,:,0]
            #eta = where(q[:,:,0] > 1.,eta,nan)
            #import pdb; pdb.set_trace()
        
            #import pdb; pdb.set_trace()
            topo = eta - h
            cutoff = 20.
            #cutoff2 = -500.
            shift = 2.84
            scale = 0.0025
            topo1 = scale*where(topo<cutoff, topo-shift, cutoff-shift)
            #topo1 = scale*where(topo>cutoff2, topo1, nan)
            eta1 = scale*where(eta<cutoff, eta-shift, cutoff-shift)
            water1 = where(h>=1.e-1, eta1, nan)
            mlab.mesh(x,y,topo1,colormap='Greens',vmin=-10*scale, vmax=scale*cutoff)
            mlab.mesh(x,y,water1,colormap='Blues',vmin=-10*scale, vmax=7*scale)
            if figno==220:
                V =  (166.63056901872105, 56.77882574597453,\
                    1.3046596144596134,\
                     array([  9.25859737,  50.21875   ,  -0.10731645]))
                V = (-32.937456325329322, 55.943603325957469,\
                  1.3046596144598517,\
                   array([  9.25859737,  50.21875   ,  -0.10731645]))
                V = (-32.937456325328981, 55.943603325957362,\
                  0.89110007134748148,\
                   array([  9.25859737,  50.21875   ,  -0.10731645]))
                V = (-36.14576828183371, 60.364412147579998,\
                    0.60863333880710002,\
                    array([  9.25859737,  50.21875   ,  -0.10731645]))
                V = (-24.488139413300885, 62.821825956355589,
                    0.6086333388071542,\
                    array([  9.25859737,  50.21875   ,  -0.10731645]))
                mlab.view(*V)
            if figno==260:
                V = (-45.733457152091233, 65.644273258202958,\
                   0.64345434573424953, \
                  array([ 14.28778982,  40.96875   ,  -0.10734103]))
                mlab.view(*V)
    t = int(frame.t)
    mlab.title('Time = %i' % t,color=(0,0,0),height=0.1,size=0.5)

def make_all_frames(figno):
    if figno==220: frames = [9,10,11,13]
    if figno==220: frames = [1,5,9,17]
    if figno==260: frames = [1,5,9,17]
    frames = [5,9,17,19]
    #if figno==260: frames = range(1,26) # for movie
    plotfiles = []
    for frameno in frames:
        plotframe2(frameno,figno)
        frstring = str(int(frameno)).zfill(3)
        name = "island-%s-frame%s.png" % (figno,frstring)
        plotfiles.append(name)
        mlab.savefig("island-%s-frame%s.png" % (figno,frstring))
        os.system("convert island-%s-frame%s.png island-%s-frame%s.eps" \
            % (figno,frstring,figno,frstring))
    #os.system("convert island-%s-frame*.png island-%s.gif" % (figno,figno))
    #make_movie(plotfiles, moviename='island-%s.html' % figno)


def rotate():
    mlab.figure(3)
    for tt in linspace(30,160,14):
        mlab.view(0,tt)
        mlab.draw()
        mlab.savefig('figure-rotate%s.png' % str(int(tt)).zfill(3))
    os.system("convert figure-rotate*.png figure.gif")

def plotframe2(frameno,figno=260,level=5):

    plotdata = ClawPlotData()
    plotdata.outdir = "_output_%i" % figno
    print "Plotting solution from ",plotdata.outdir
    plotdata = setplot(plotdata)
    try:
        frame = plotdata.getframe(frameno)
    except:
        print "Unable to get frame"
        return
    mlab.figure(figno,bgcolor=(1,1,1))
    mlab.clf()

    theta_island = figno
    (xisland,yisland) = latlong(1600.e3, theta_island, 40., Rearth)
    (xi2,yi2) = latlong(1601.e3, theta_island, 40., Rearth)
    xisland = xisland * cos(pi*yisland/180.)
    xi2 = xi2 * cos(pi*yi2/180.)
    xydist = sqrt((xi2-xisland)**2 + (yi2-yisland)**2)
    xnormal = (xi2-xisland) / xydist
    ynormal = (yi2-yisland) / xydist

    for grid in frame.grids:
        if grid.level == level:

            y = grid.c_center[1]
            # scale x by cos(y) to adjust for sphere:
            x = grid.c_center[0] * cos(pi*y/180.)
            xn = (x-xisland)*xnormal + (y-yisland)*ynormal
            yn = -(x-xisland)*ynormal + (y-yisland)*xnormal
            xn = 111.*xn  # kilometers / degree
            yn = 111.*yn  # kilometers / degree
            print "xn range: ",xn.min(),xn.max()
            print "yn range: ",yn.min(),yn.max()
            
            q = grid.q
            eta = q[:,:,3]
            h = q[:,:,0]
            #eta = where(q[:,:,0] > 1.,eta,nan)
            #import pdb; pdb.set_trace()
        
            topo = eta - h
            cutoff = 20.
            #cutoff2 = -500.
            shift = 0.
            scale = 0.2
            topo1 = scale*where(topo<cutoff, topo-shift, cutoff-shift)
            #topo1 = scale*where(topo>cutoff2, topo1, nan)
            eta1 = scale*where(eta<cutoff, eta-shift, cutoff-shift)
            water1 = where(h>=1.e-1, eta1, nan)
            mlab.mesh(xn,yn,topo1,colormap='Greens',vmin=-10*scale, vmax=scale*cutoff)
            mlab.mesh(xn,yn,water1,colormap='Blues',vmin=-10*scale, vmax=7*scale)
            mlab.view(-95,70,70,[.04,-.4,-8])
    t = int(frame.t)
    #mlab.title('Time = %i' % t,color=(0,0,0),height=0.1,size=0.5)
